import fcntl
import struct
import time

# INA231 register addresses
INA231_BUS_VOLTAGE = 0x02
INA231_CALIBRATION = 0x05

# Configuration constants
I2C_BUS = 3            # M0205 + M0206 --> /dev/i2c-3
I2C_ADDR = 0x44        # INA231 I2C address
I2C_DEV = f"/dev/i2c-{I2C_BUS}"
SHUNT_RESISTOR = 0.0005  # 500 µOhm
MAX_EXPECTED_CURRENT = 10.0  # Example max current in Amps
CURRENT_LSB = MAX_EXPECTED_CURRENT / 32768.0
CALIBRATION = int(0.00512 / (CURRENT_LSB * SHUNT_RESISTOR))

# I2C helper functions
def write_register(fd, reg, value):
    data = struct.pack('>BH', reg, value)
    fcntl.ioctl(fd, 0x0703, I2C_ADDR)  # I2C_SLAVE
    fd.write(data)

def read_register(fd, reg):
    fcntl.ioctl(fd, 0x0703, I2C_ADDR)  # I2C_SLAVE
    fd.write(struct.pack('B', reg))
    raw = fd.read(2)
    return (raw[0] << 8) | raw[1]

def read_voltage(fd):
    bus_voltage_raw = read_register(fd, INA231_BUS_VOLTAGE)
    return bus_voltage_raw * 0.00125  # Bus voltage LSB = 1.25mV

def main():
    with open(I2C_DEV, 'r+b', buffering=0) as fd:
        write_register(fd, INA231_CALIBRATION, CALIBRATION)

        try:
            while True:
                voltage = read_voltage(fd)
                print(f"Bus Voltage: {voltage:.3f} V")
                time.sleep(1)
                break
        except KeyboardInterrupt:
            print("Exiting...")

if __name__ == "__main__":
    main()
